@@ -11,11 +11,12 @@ import sys
 from resilient_sdk.cmds import (CmdClone, CmdCodegen, CmdDev, CmdDocgen,
                                 CmdExtPackage, CmdExtract, CmdValidate)
 from resilient_sdk.util import sdk_helpers
+from resilient_sdk.util import constants
 from resilient_sdk.util.sdk_argparse import SDKArgumentParser
 from resilient_sdk.util.sdk_exception import SDKException
 
 # Setup logging
-LOG = logging.getLogger(sdk_helpers.LOGGER_NAME)
+LOG = logging.getLogger(constants.LOGGER_NAME)
 LOG.setLevel(logging.INFO)
 LOG.addHandler(logging.StreamHandler())
 
@@ -30,7 +31,7 @@ def get_main_app_parser():
     # Define main parser object
     # We use SDKArgumentParser which overwrites the 'error' method
     parser = SDKArgumentParser(
-        prog=sdk_helpers.SDK_PACKAGE_NAME,
+        prog=constants.SDK_PACKAGE_NAME,
         description="""Python SDK for developing IBM SOAR Apps that
         provides various subcommands to help with development""",
         epilog="For support, please visit ibm.biz/soarcommunity")
@@ -80,7 +81,7 @@ def main():
     os.system("")
 
     # See if RES_SDK_DEV environment var is set
-    sdk_dev = sdk_helpers.is_env_var_set(sdk_helpers.ENV_VAR_DEV)
+    sdk_dev = sdk_helpers.is_env_var_set(constants.ENV_VAR_DEV)
 
     # Get main parser object
     parser = get_main_app_parser()
@@ -88,18 +89,18 @@ def main():
     # Get sub_parser object, its dest is cmd
     sub_parser = get_main_app_sub_parser(parser)
 
+    if sdk_dev:
+        # Add 'dev' command if environment var set
+        cmd_dev = CmdDev(sub_parser)
+        LOG.info("{0}Running SDK in Developer Mode{0}".format(constants.LOG_DIVIDER))
+
     # Add any subcommands to main app parser here
+    cmd_validate = CmdValidate(sub_parser)
     cmd_codegen = CmdCodegen(sub_parser)
     cmd_clone = CmdClone(sub_parser)
     cmd_docgen = CmdDocgen(sub_parser)
     cmd_extract = CmdExtract(sub_parser)
-    cmd_ext_package = CmdExtPackage(sub_parser)
-
-    if sdk_dev:
-        # Add 'dev' command if environment var set
-        cmd_dev = CmdDev(sub_parser)
-        cmd_validate = CmdValidate(sub_parser)
-        LOG.info("{0}Running SDK in Developer Mode{0}".format(sdk_helpers.LOG_DIVIDER))
+    cmd_ext_package = CmdExtPackage(sub_parser, cmd_validate=cmd_validate)
 
     try:
         # Parse the arguments
@@ -114,7 +115,7 @@ def main():
         main_cmd = sdk_helpers.get_main_cmd()
 
         LOG.error(err)
-        LOG.info("{0}".format(sdk_helpers.LOG_DIVIDER))
+        LOG.info("{0}".format(constants.LOG_DIVIDER))
 
         # Print specifc usage for that cmd for these errors
         if "too few arguments" in err.message or "no subcommand provided" in err.message:
@@ -133,7 +134,7 @@ def main():
             elif main_cmd == cmd_ext_package.CMD_NAME:
                 cmd_ext_package.parser.print_usage()
 
-            elif sdk_dev and main_cmd == cmd_validate.CMD_NAME:
+            elif main_cmd == cmd_validate.CMD_NAME:
                 cmd_validate.parser.print_usage()
 
             elif sdk_dev and main_cmd == cmd_dev.CMD_NAME:
@@ -166,9 +167,7 @@ def main():
     elif args.cmd == cmd_ext_package.CMD_NAME:
         cmd_ext_package.execute_command(args)
 
-    elif sdk_dev and args.cmd == cmd_validate.CMD_NAME:
-        # TODO: remove sdk_validate_enabled check once validate is ready for use
-        # functionality is currently hidden behind sdk_validate_enabled env var 
+    elif args.cmd == cmd_validate.CMD_NAME:
         cmd_validate.execute_command(args)
     elif sdk_dev and args.cmd == cmd_dev.CMD_NAME:
         cmd_dev.execute_command(args)
