@@ -4,16 +4,20 @@
 
 """ Implementation of `resilient-sdk codegen` """
 
+import json
 import logging
 import os
 import re
 import shutil
+
 from resilient import ensure_unicode
 from resilient_sdk.cmds.base_cmd import BaseCmd
-from resilient_sdk.util.sdk_exception import SDKException
-from resilient_sdk.util.resilient_objects import ResilientObjMap
+from resilient_sdk.util import constants
 from resilient_sdk.util import package_file_helpers as package_helpers
-from resilient_sdk.util import sdk_helpers, constants
+from resilient_sdk.util import sdk_helpers
+from resilient_sdk.util.resilient_objects import ResilientObjMap
+from resilient_sdk.util.sdk_exception import SDKException
+from resilient_sdk.util.sdk_genson_overwrites import main_genson_builder_overwrites, CustomSchemaBuilder
 
 # Get the same logger object that is used in app.py
 LOG = logging.getLogger(constants.LOGGER_NAME)
@@ -27,7 +31,9 @@ class CmdCodegen(BaseCmd):
     CMD_USAGE = """
     $ resilient-sdk codegen -p <name_of_package> -m 'fn_custom_md' --rule 'Rule One' 'Rule Two' -i 'custom incident type'
     $ resilient-sdk codegen -p <name_of_package> -m 'fn_custom_md' -c '/usr/custom_app.config'
-    $ resilient-sdk codegen -p <path_current_package> --reload --workflow 'new_wf_to_add'"""
+    $ resilient-sdk codegen -p <path_current_package> --reload --workflow 'new_wf_to_add'
+    $ resilient-sdk codegen -p <path_current_package> --gather-results
+    $ resilient-sdk codegen -p <path_current_package> --gather-results '/usr/custom_app.log' -f 'func_one' 'func_two'"""
     CMD_DESCRIPTION = CMD_HELP
     CMD_ADD_PARSERS = ["app_config_parser", "res_obj_parser", "io_parser"]
 
@@ -45,10 +51,23 @@ class CmdCodegen(BaseCmd):
                                  action="store_true",
                                  help="Reload customizations and create new customize.py")
 
+        self.parser.add_argument(constants.SUB_CMD_OPT_GATHER_RESULTS,
+                                 action="store",
+                                 nargs="?",
+                                 const=constants.PATH_RES_DEFAULT_LOG_FILE,
+                                 help="Uses the log file specified or if no path specified use the default at '~/.resilient/logs/app.log' to try gather results. Only Python >= 3.6 supported")
+
     def execute_command(self, args):
         LOG.debug("called: CmdCodegen.execute_command()")
 
-        if args.reload:
+        if args.gather_results:
+            if not args.package:
+                raise SDKException("'-p' must be specified when using '{0}'".format(constants.SUB_CMD_OPT_GATHER_RESULTS))
+
+            SDKException.command_ran = "{0} {1}".format(self.CMD_NAME, constants.SUB_CMD_OPT_GATHER_RESULTS)
+            self._get_results_from_log_file(args)
+
+        elif args.reload:
             if not args.package:
                 raise SDKException("'-p' must be specified when using '--reload'")
 
@@ -219,10 +238,15 @@ class CmdCodegen(BaseCmd):
         # Add to that dict
         ps_dict[package_helpers.BASE_NAME_PAYLOAD_SAMPLES_SCHEMA] = (u"{0}/blank.json.jinja2".format(package_helpers.PATH_TEMPLATE_PAYLOAD_SAMPLES), jinja_data)
         ps_dict[package_helpers.BASE_NAME_PAYLOAD_SAMPLES_EXAMPLE] = (u"{0}/blank.json.jinja2".format(package_helpers.PATH_TEMPLATE_PAYLOAD_SAMPLES), jinja_data)
-        ps_dict[package_helpers.BASE_NAME_PAYLOAD_SAMPLES_EX_SUCCESS] = (u"{0}/mock_json_expectation_success.json.jinja2".format(package_helpers.PATH_TEMPLATE_PAYLOAD_SAMPLES), jinja_data)
-        ps_dict[package_helpers.BASE_NAME_PAYLOAD_SAMPLES_EP_SUCCESS] = (u"{0}/blank.json.jinja2".format(package_helpers.PATH_TEMPLATE_PAYLOAD_SAMPLES), jinja_data)
-        ps_dict[package_helpers.BASE_NAME_PAYLOAD_SAMPLES_EX_FAIL] = (u"{0}/mock_json_expectation_fail.json.jinja2".format(package_helpers.PATH_TEMPLATE_PAYLOAD_SAMPLES), jinja_data)
-        ps_dict[package_helpers.BASE_NAME_PAYLOAD_SAMPLES_EP_FAIL] = (u"{0}/blank.json.jinja2".format(package_helpers.PATH_TEMPLATE_PAYLOAD_SAMPLES), jinja_data)
+
+        # TODO: re-enable this code when we have logic to use mock_server files
+        """
+        if sdk_helpers.is_env_var_set(constants.ENV_VAR_DEV):
+            ps_dict[package_helpers.BASE_NAME_PAYLOAD_SAMPLES_EX_SUCCESS] = (u"{0}/mock_json_expectation_success.json.jinja2".format(package_helpers.PATH_TEMPLATE_PAYLOAD_SAMPLES), jinja_data)
+            ps_dict[package_helpers.BASE_NAME_PAYLOAD_SAMPLES_EP_SUCCESS] = (u"{0}/blank.json.jinja2".format(package_helpers.PATH_TEMPLATE_PAYLOAD_SAMPLES), jinja_data)
+            ps_dict[package_helpers.BASE_NAME_PAYLOAD_SAMPLES_EX_FAIL] = (u"{0}/mock_json_expectation_fail.json.jinja2".format(package_helpers.PATH_TEMPLATE_PAYLOAD_SAMPLES), jinja_data)
+            ps_dict[package_helpers.BASE_NAME_PAYLOAD_SAMPLES_EP_FAIL] = (u"{0}/blank.json.jinja2".format(package_helpers.PATH_TEMPLATE_PAYLOAD_SAMPLES), jinja_data)
+        """
 
     @staticmethod
     def _gen_function(args):
@@ -255,7 +279,7 @@ class CmdCodegen(BaseCmd):
             org_export = sdk_helpers.read_local_exportfile(args.exportfile)
 
         else:
-            # Instantiate connection to the Resilient Appliance
+            # Instantiate connection to SOAR
             res_client = sdk_helpers.get_resilient_client(path_config_file=args.config)
 
             # Generate + get latest export from Resilient Server
@@ -272,7 +296,8 @@ class CmdCodegen(BaseCmd):
                                                  datatables=args.datatable,
                                                  tasks=args.task,
                                                  scripts=args.script,
-                                                 incident_types=args.incidenttype)
+                                                 incident_types=args.incidenttype,
+                                                 playbooks=args.playbook)
 
         # Get 'minified' version of the export. This is used in customize.py
         jinja_data["export_data"] = sdk_helpers.minify_export(org_export,
@@ -286,7 +311,8 @@ class CmdCodegen(BaseCmd):
                                                               tasks=sdk_helpers.get_object_api_names(ResilientObjMap.TASKS, jinja_data.get("tasks")),
                                                               phases=sdk_helpers.get_object_api_names(ResilientObjMap.PHASES, jinja_data.get("phases")),
                                                               scripts=sdk_helpers.get_object_api_names(ResilientObjMap.SCRIPTS, jinja_data.get("scripts")),
-                                                              incident_types=sdk_helpers.get_object_api_names(ResilientObjMap.INCIDENT_TYPES, jinja_data.get("incident_types")))
+                                                              incident_types=sdk_helpers.get_object_api_names(ResilientObjMap.INCIDENT_TYPES, jinja_data.get("incident_types")),
+                                                              playbooks=sdk_helpers.get_object_api_names(ResilientObjMap.PLAYBOOKS, jinja_data.get("playbooks")))
 
         # Add package_name to jinja_data
         jinja_data["package_name"] = package_name
@@ -351,9 +377,7 @@ class CmdCodegen(BaseCmd):
         # If there are Functions, add a 'tests' and a 'payload_samples' directory (if in dev mode)
         if jinja_data.get("functions"):
             package_mapping_dict["tests"] = {}
-
-            if sdk_helpers.is_env_var_set(constants.ENV_VAR_DEV):
-                package_mapping_dict["payload_samples"] = {}
+            package_mapping_dict[package_helpers.BASE_NAME_PAYLOAD_SAMPLES_DIR] = {}
 
         # Get a list of function names in export.
         fn_names = [f.get(ResilientObjMap.FUNCTIONS) for f in jinja_data.get("functions")]
@@ -383,10 +407,8 @@ class CmdCodegen(BaseCmd):
             # Add to 'tests' directory
             package_mapping_dict["tests"][u"test_{0}".format(file_name)] = ("tests/test_function.py.jinja2", f)
 
-            # See if RES_SDK_DEV environment var is set
-            if sdk_helpers.is_env_var_set(constants.ENV_VAR_DEV):
-                # Add a 'payload_samples/fn_name' directory and the files to it
-                CmdCodegen.add_payload_samples(package_mapping_dict, fn_name, f)
+            # Add a 'payload_samples/fn_name' directory and the files to it
+            CmdCodegen.add_payload_samples(package_mapping_dict, fn_name, f)
 
         # Get a list of workflow names in export.
         wf_names = [w.get(ResilientObjMap.WORKFLOWS) for w in jinja_data.get("workflows")]
@@ -439,14 +461,9 @@ class CmdCodegen(BaseCmd):
         # Ensure the package directory exists and we have WRITE access
         sdk_helpers.validate_dir_paths(os.W_OK, path_package)
 
-        # Generate path to setup.py file + validate we have permissions to read it
         path_setup_py_file = os.path.join(path_package, package_helpers.BASE_NAME_SETUP_PY)
-        sdk_helpers.validate_file_paths(os.R_OK, path_setup_py_file)
 
-        # Parse the setup.py file
-        setup_py_attributes = package_helpers.parse_setup_py(path_setup_py_file, package_helpers.SUPPORTED_SETUP_PY_ATTRIBUTE_NAMES)
-
-        package_name = setup_py_attributes.get("name")
+        package_name = package_helpers.get_package_name(path_package)
 
         if not sdk_helpers.is_valid_package_name(package_name):
             raise SDKException(u"'{0}' is not a valid package name. 'name' attribute in setup.py file is not valid or not specified".format(package_name))
@@ -500,7 +517,8 @@ class CmdCodegen(BaseCmd):
                 ("incidenttype", "incident_types"),
                 ("datatable", "datatables"),
                 ("task", "automatic_tasks"),
-                ("script", "scripts")
+                ("script", "scripts"),
+                ("playbook", "playbooks")
             ]
 
             # Merge old_params with new params specified on command line
@@ -528,3 +546,89 @@ class CmdCodegen(BaseCmd):
             if not os.path.isfile(path_export_res) and path_export_res_bak:
                 LOG.info(u"An error occurred. Renaming export.res.bak to export.res")
                 sdk_helpers.rename_file(path_export_res_bak, package_helpers.BASE_NAME_LOCAL_EXPORT_RES)
+
+    @classmethod
+    def _get_results_from_log_file(cls, args):
+        """
+        - Gets all function names from the payload_samples directory
+        - Traverses the file at the path specified by args.gather_results (in a reversed order)
+        - Looks for lines containing ``[<fn_name>] Result: {'version': 2.0, 'success': True...``
+        - Parses it and generates an output_json_example.json and output_json_schema.json file for each ``Result`` found
+        - Uses the libary ``genson`` to generate the JSON schema from a Python Dictionary
+
+        :param args: (required) the cmd line arguments
+        :type args: argparse.ArgumentParser
+        :raises: an SDKException if args.package is not a valid path
+        """
+
+        # Check if Python >= MIN_SUPPORTED_PY_VERSION
+        if not sdk_helpers.is_python_min_supported_version(constants.ERROR_WRONG_PYTHON_VERSION):
+            raise SDKException(constants.ERROR_WRONG_PYTHON_VERSION)
+
+        path_package = os.path.abspath(args.package)
+        path_log_file = args.gather_results
+        path_payload_samples_dir = os.path.join(path_package, package_helpers.BASE_NAME_PAYLOAD_SAMPLES_DIR)
+
+        LOG.debug("\nPath to project: %s", path_package)
+
+        sdk_helpers.validate_dir_paths(os.W_OK, path_package)
+
+        package_name = package_helpers.get_package_name(path_package)
+
+        LOG.info("'codegen %s' started for '%s'", constants.SUB_CMD_OPT_GATHER_RESULTS, package_name)
+        try:
+
+            sdk_helpers.validate_dir_paths(os.W_OK, path_payload_samples_dir)
+
+        except SDKException as e:
+
+            if constants.ERROR_NOT_FIND_DIR in e.message:
+                LOG.warning("WARNING: no '%s' found. Running 'codegen --reload' to create the default missing files\n%s", package_helpers.BASE_NAME_PAYLOAD_SAMPLES_DIR, constants.LOG_DIVIDER)
+                args.reload = True
+                cls._reload_package(args)
+                LOG.warning(constants.LOG_DIVIDER)
+
+            else:
+                raise e
+
+        functions_that_need_payload_samples = args.function if args.function else os.listdir(path_payload_samples_dir)
+
+        results_scraped = sdk_helpers.scrape_results_from_log_file(path_log_file)
+
+        for fn_name in functions_that_need_payload_samples:
+
+            fn_results = results_scraped.get(fn_name)
+
+            if not fn_results:
+                package_helpers.color_output("WARNING: No results could be found for '{0}' in '{1}'".format(fn_name, path_log_file), constants.VALIDATE_LOG_LEVEL_WARNING, do_print=True)
+                continue
+
+            LOG.info("Results found for '[%s]'", fn_name)
+
+            path_payload_samples_fn_name = os.path.join(path_payload_samples_dir, fn_name)
+            path_output_json_example = os.path.join(path_payload_samples_fn_name, package_helpers.BASE_NAME_PAYLOAD_SAMPLES_EXAMPLE)
+            path_output_json_schema = os.path.join(path_payload_samples_fn_name, package_helpers.BASE_NAME_PAYLOAD_SAMPLES_SCHEMA)
+
+            path_output_json_example_bak = sdk_helpers.rename_to_bak_file(path_output_json_example)
+            path_output_json_schema_bak = sdk_helpers.rename_to_bak_file(path_output_json_schema)
+
+            try:
+                LOG.debug("Writing JSON example file for '%s' to '%s'", fn_name, path_output_json_example)
+                sdk_helpers.write_file(path_output_json_example, json.dumps(fn_results, indent=2))
+
+                LOG.debug("Writing JSON schema file for '%s' to '%s'", fn_name, path_output_json_schema)
+                builder = CustomSchemaBuilder(schema_uri=constants.CODEGEN_JSON_SCHEMA_URI)
+                main_genson_builder_overwrites(builder)
+                builder.add_object(fn_results)
+                sdk_helpers.write_file(path_output_json_schema, builder.to_json(indent=2))
+
+            finally:
+                if not os.path.isfile(path_output_json_example) and path_output_json_example_bak:
+                    LOG.info(u"An error occurred. Renaming %s.bak to %s", package_helpers.BASE_NAME_PAYLOAD_SAMPLES_EXAMPLE, package_helpers.BASE_NAME_PAYLOAD_SAMPLES_EXAMPLE)
+                    sdk_helpers.rename_file(path_output_json_example_bak, package_helpers.BASE_NAME_PAYLOAD_SAMPLES_EXAMPLE)
+
+                if not os.path.isfile(path_output_json_schema) and path_output_json_schema_bak:
+                    LOG.info(u"An error occurred. Renaming %s.bak to %s", package_helpers.BASE_NAME_PAYLOAD_SAMPLES_SCHEMA, package_helpers.BASE_NAME_PAYLOAD_SAMPLES_SCHEMA)
+                    sdk_helpers.rename_file(path_output_json_schema_bak, package_helpers.BASE_NAME_PAYLOAD_SAMPLES_SCHEMA)
+
+        LOG.info("'codegen %s' complete for '%s'", constants.SUB_CMD_OPT_GATHER_RESULTS, package_name)
