@@ -7,12 +7,15 @@
 import logging
 import os
 import shutil
+
 from resilient import ensure_unicode
 from resilient_sdk.cmds.base_cmd import BaseCmd
-from resilient_sdk.util.sdk_exception import SDKException
-from resilient_sdk.util import sdk_helpers, constants
+from resilient_sdk.util import constants
 from resilient_sdk.util import package_file_helpers as package_helpers
-from resilient_sdk.util.resilient_objects import IGNORED_INCIDENT_FIELDS, ResilientObjMap
+from resilient_sdk.util import sdk_helpers
+from resilient_sdk.util.resilient_objects import (IGNORED_INCIDENT_FIELDS,
+                                                  ResilientObjMap)
+from resilient_sdk.util.sdk_exception import SDKException
 
 # Get the same logger object that is used in app.py
 LOG = logging.getLogger(constants.LOGGER_NAME)
@@ -90,6 +93,7 @@ class CmdDocgen(BaseCmd):
             the_function["inputs"] = cls._get_fn_input_details(fn)
             the_function["message_destination"] = fn.get("destination_handle", "")
             the_function["workflows"] = fn.get("workflows", [])
+            the_function["x_api_name"] = fn.get("x_api_name", "")
 
             scripts_found = False
             pre_script = None
@@ -176,6 +180,26 @@ class CmdDocgen(BaseCmd):
 
         return return_list
 
+    @staticmethod
+    def _get_playbook_details(playbooks):
+        """Return a List of all Playbooks which are Dictionaries with
+        the attributes: api_name, name, object_type, status and description"""
+
+        return_list = []
+
+        for playbook in playbooks:
+            the_playbook = {}
+
+            the_playbook["api_name"] = playbook.get("x_api_name")
+            the_playbook["name"] = playbook.get("display_name")
+            the_playbook["object_type"] = playbook.get("object_type", "")
+            the_playbook["status"] = playbook.get("status", "")
+            the_playbook["description"] = playbook.get("description", {}).get("content", "")
+
+            return_list.append(the_playbook)
+
+        return return_list
+
     @staticmethod
     def _get_datatable_details(datatables):
         """Return a List of all Data Tables which are Dictionaries with
@@ -286,6 +310,7 @@ class CmdDocgen(BaseCmd):
         path_config_py_file = os.path.join(path_to_src, package_name, package_helpers.PATH_CONFIG_PY)
         path_readme = os.path.join(path_to_src, package_helpers.BASE_NAME_README)
         path_screenshots_dir = os.path.join(path_to_src, package_helpers.PATH_SCREENSHOTS)
+        path_payload_samples_dir = os.path.join(path_to_src, package_helpers.BASE_NAME_PAYLOAD_SAMPLES_DIR)
 
         # Ensure we have read permissions for each required file and the file exists
         sdk_helpers.validate_file_paths(os.R_OK, path_setup_py_file, path_customize_py_file, path_config_py_file)
@@ -325,7 +350,8 @@ class CmdDocgen(BaseCmd):
                                                       artifact_types=sdk_helpers.get_object_api_names(ResilientObjMap.INCIDENT_ARTIFACT_TYPES, customize_py_import_def.get("incident_artifact_types")),
                                                       datatables=sdk_helpers.get_object_api_names(ResilientObjMap.DATATABLES, customize_py_import_def.get("types")),
                                                       tasks=sdk_helpers.get_object_api_names(ResilientObjMap.TASKS, customize_py_import_def.get("automatic_tasks")),
-                                                      scripts=sdk_helpers.get_object_api_names(ResilientObjMap.SCRIPTS, customize_py_import_def.get("scripts")))
+                                                      scripts=sdk_helpers.get_object_api_names(ResilientObjMap.SCRIPTS, customize_py_import_def.get("scripts")),
+                                                      playbooks=sdk_helpers.get_object_api_names(ResilientObjMap.PLAYBOOKS, customize_py_import_def.get("playbooks", [])))
 
         # Lists we use in Jinja Templates
         jinja_functions = self._get_function_details(import_def_data.get("functions", []), import_def_data.get("workflows", []))
@@ -334,12 +360,33 @@ class CmdDocgen(BaseCmd):
         jinja_datatables = self._get_datatable_details(import_def_data.get("datatables", []))
         jinja_custom_fields = self._get_custom_fields_details(import_def_data.get("fields", []))
         jinja_custom_artifact_types = self._get_custom_artifact_details(import_def_data.get("artifact_types", []))
+        jinja_playbooks = self._get_playbook_details(import_def_data.get("playbooks", []))
 
         # Other variables for Jinja Templates
         package_name_dash = package_name.replace("_", "-")
         server_version = customize_py_import_def.get("server_version", {})
         supported_app = sdk_helpers.does_url_contain(setup_py_attributes.get("url", ""), "ibm.com/mysupport")
 
+        # See if a payload_samples dir exists and use the contents for function results
+        try:
+            sdk_helpers.validate_dir_paths(os.R_OK, path_payload_samples_dir)
+
+            for f in jinja_functions:
+                fn_name = f.get("x_api_name")
+                path_payload_samples_fn_name = os.path.join(path_payload_samples_dir, fn_name)
+                path_output_json_example = os.path.join(path_payload_samples_fn_name, package_helpers.BASE_NAME_PAYLOAD_SAMPLES_EXAMPLE)
+
+                try:
+                    sdk_helpers.validate_file_paths(os.R_OK, path_output_json_example)
+                    f["results"] = sdk_helpers.read_json_file(path_output_json_example)
+                except SDKException as e:
+                    sdk_helpers.handle_file_not_found_error(e, u"Error getting results. No '{0}' file found for '{1}'.".format(
+                        package_helpers.BASE_NAME_PAYLOAD_SAMPLES_EXAMPLE, fn_name))
+
+        except SDKException as e:
+            sdk_helpers.handle_file_not_found_error(e, u"Error getting results. No '{0}' directory found.".format(
+                package_helpers.BASE_NAME_PAYLOAD_SAMPLES_EXAMPLE))
+
         LOG.info("Rendering README for %s", package_name_dash)
 
         # Render the README Jinja2 Templeate with parameters
@@ -363,6 +410,7 @@ class CmdDocgen(BaseCmd):
             "datatables": jinja_datatables,
             "custom_fields": jinja_custom_fields,
             "custom_artifact_types": jinja_custom_artifact_types,
+            "playbooks": jinja_playbooks,
             "placeholder_string": constants.DOCGEN_PLACEHOLDER_STRING
         })
 
